@@ -1,26 +1,32 @@
 """
 Operation and axis aligmnent
 """
+from __future__ import absolute_import
 import numpy as np
+import warnings
 
-from align import align_dims, align_axes
-from tools import is_DimArray
+from dimarray.tools import is_DimArray
+from .align import  align_dims, align as align_axes
+from .axes import Axes
 
 def operation(func, o1, o2, reindex=True, broadcast=True, constructor=None):
-    """ operation on LaxArray objects
+    """ binary operation involving a DimArray objects
 
     Parameters
     ----------
-        func        : operator
-        o1            : LHS operand: DimArray
-        o2            : RHS operand: at least: be convertible by np.array())
-        align, optional: if True, use pandas to align the axes
+    func : operator
+    o1 : LHS operand: DimArray
+    o2 : RHS operand: at least: be convertible by np.array())
+    align : bool, optional
+        if True, use pandas to align the axes
+    constructor : class Constructor, optional
+        if None, o1's class constructor (o1._constructor) is used instead
 
     Returns
     -------
-        values: array values
-        dims : dimension names
+    DimArray instance
     """
+
     if constructor is None:
         constructor = o1._constructor
 
@@ -40,6 +46,11 @@ def operation(func, o1, o2, reindex=True, broadcast=True, constructor=None):
 
     # both objects are dimarrays
 
+    # check grid mapping and emit a warning if mismatch
+    if hasattr(o1, 'grid_mapping') and hasattr(o2, 'grid_mapping') \
+            and o1.grid_mapping != o2.grid_mapping:
+                warnings.warn("binary op : grid mappings mismatch")
+
     # Align axes by re-indexing
     if reindex:
         o1, o2 = align_axes(o1, o2)
@@ -49,12 +60,13 @@ def operation(func, o1, o2, reindex=True, broadcast=True, constructor=None):
         o1, o2 = align_dims(o1, o2)
 
     # make the new axes
-    newaxes = o1.axes.copy()
-
-    # ...make sure no singleton value is included
-    for i, ax in enumerate(newaxes):
+    newaxes = Axes()
+    for i, ax in enumerate(o1.axes):
         if ax.values[0] is None:
-            newaxes[i] = o2.axes[ax.name]
+            # ...make sure no singleton value is included
+            newaxes.append(o2.axes[ax.name].copy())
+        else:
+            newaxes.append(ax.copy())
 
     res = func(o1.values, o2.values)
 
