@@ -6,8 +6,10 @@ import numpy as np
 from collections import OrderedDict
 import functools
 
-from axes import Axis, Axes, GroupedAxis
-from tools import is_DimArray
+from axes import Axis, Axes, MultiAxis
+#from dimarray.tools import is_DimArray
+import dimarray as da
+from dimarray.tools import deprecated_func
 
 #
 # Broadcast
@@ -32,10 +34,9 @@ def broadcast(self, other):
     >>> lon = np.linspace(10, 30, 2)
     >>> lat = np.linspace(10, 50, 3)
     >>> time = np.arange(1950,1955)
-    >>> ts = da.DimArray.from_kw(np.arange(5), time=time)
-    >>> cube = da.DimArray.from_kw(np.zeros((3,2,5)), lon=lon, lat=lat, time=time)  # lat x lon x time
+    >>> ts = da.DimArray(np.arange(5), axes=[time], dims=['time'])
+    >>> cube = da.DimArray(np.zeros((3,2,5)), axes=[('lat',lat), ('lon',lon), ('time',time)])  # lat x lon x time
     >>> cube.axes  
-    dimensions: 'lat', 'lon', 'time'
     0 / lat (3): 10.0 to 50.0
     1 / lon (2): 10.0 to 30.0
     2 / time (5): 1950 to 1954
@@ -45,7 +46,6 @@ def broadcast(self, other):
     >>> ts3D = ts.broadcast(cube) #  lat x lon x time
     >>> ts3D
     dimarray: 30 non-null elements (0 null)
-    dimensions: 'lat', 'lon', 'time'
     0 / lat (3): 10.0 to 50.0
     1 / lon (2): 10.0 to 30.0
     2 / time (5): 1950 to 1954
@@ -63,7 +63,7 @@ def broadcast(self, other):
         newaxes = other
 
     # Or as DimArray
-    elif is_DimArray(other):
+    elif isinstance(other, da.DimArray):
          newaxes = other.axes
 
     # Or as OrderedDict of axis names, axis values
@@ -97,9 +97,18 @@ def transpose(self, *dims):
     
     Analogous to numpy, but also allows axis names
 
+    Parameters
+    ----------
+    *dims : int or str
+        variable list of dimensions
+
+    Returns
+    -------
+    transposed_array : DimArray
+
     See also
     ---------
-    reshape, group, ungroup, newaxis
+    reshape, flatten, unflatten, newaxis
 
     Examples
     --------
@@ -107,14 +116,12 @@ def transpose(self, *dims):
     >>> a = da.DimArray(np.zeros((2,3)), ['x0','x1'])
     >>> a          
     dimarray: 6 non-null elements (0 null)
-    dimensions: 'x0', 'x1'
     0 / x0 (2): 0 to 1
     1 / x1 (3): 0 to 2
     array([[ 0.,  0.,  0.],
            [ 0.,  0.,  0.]])
     >>> a.T       
     dimarray: 6 non-null elements (0 null)
-    dimensions: 'x1', 'x0'
     0 / x1 (3): 0 to 2
     1 / x0 (2): 0 to 1
     array([[ 0.,  0.],
@@ -141,10 +148,12 @@ def transpose(self, *dims):
 
     result = self.values.transpose(newshape)
     newaxes = [self.axes[i] for i in newshape]
-    return self._constructor(result, newaxes)
+    return self._constructor(result, newaxes, **self.attrs)
 
 def swapaxes(self, axis1, axis2):
-    """ analogous to numpy's swapaxes, but can provide axes by name
+    """ Swap two axes
+    
+    analogous to numpy's swapaxes, but can provide axes by name
 
     Parameters
     ----------
@@ -153,7 +162,7 @@ def swapaxes(self, axis1, axis2):
 
     Returns
     -------
-    DimArray
+    transposed_array : DimArray
 
     Examples
     --------
@@ -179,27 +188,69 @@ def swapaxes(self, axis1, axis2):
             newshape.append(i)
     return transpose(self, newshape)
 
+def rollaxis(self, axis, start=0):
+    """ Roll the specified axis backwards, until it lies in a given position.
+
+    Parameters
+    ----------
+    axis : int or str
+        The axis to roll backwards.  The positions of the other axes do not
+        change relative to one another.
+    start : int, optional
+        The axis is rolled until it lies before this position.  The default,
+        0, results in a "complete" roll.
+
+    Returns
+    -------
+    res : DimArray instance
+        Output array.
+
+    See Also
+    --------
+    roll : Roll the elements of an array by a number of positions along a
+        given axis.  (TODO)
+
+    Examples
+    --------
+    >>> import dimarray as da
+    >>> a = da.ones(shape=(3,4,5,6))
+    >>> a.rollaxis(3, 1).shape
+    (3, 6, 4, 5)
+    >>> a.rollaxis(2).shape
+    (5, 3, 4, 6)
+    >>> a.rollaxis(1, 4).shape
+    (3, 5, 6, 4)
+    """
+    axis, _ = self._get_axis_info(axis) # position
+
+    # use numpy rollaxis on an empty array to determine the shape
+    fake = np.ones(range(self.ndim)) # first is 0 ==> empty
+    newshape = np.rollaxis(fake, axis, start).shape
+
+    return self.transpose(newshape)
+
 #
 # Repeat the array along *existing* axis
 #
 def repeat(self, values, axis=None):
-    """ expand the array along axis: analogous to numpy's repeat
-
-    Signature: repeat(values=None, axis=None, **kwaxes)
-
+    """ expand the array along an existing axis
+    
     Parameters
     ----------
-        values  : integer (size of new axis) or ndarray (values  of new axis) 
-                  or Axis object
-        axis    : int or str (refer to the dimension along which to repeat)
+    values : int or ndarray or Axis instance
+        int: size of new axis
+        ndarray: values  of new axis 
+    axis : int or str 
+        refer to the dimension along which to repeat
 
-        **kwaxes: alternatively, axes may be passed as keyword arguments 
+    **kwaxes : key-word arguments
+        alternatively, axes may be passed as keyword arguments 
 
     Returns
     -------
-        DimArray
+    DimArray
 
-    Sea Also
+    See Also
     --------
     newaxis
 
@@ -211,7 +262,6 @@ def repeat(self, values, axis=None):
 
     >>> a2d.repeat(2, axis="lon")  
     dimarray: 6 non-null elements (0 null)
-    dimensions: 'time', 'lon'
     0 / time (3): 1950.0 to 1952.0
     1 / lon (2): 0 to 1
     array([[0, 0],
@@ -220,7 +270,6 @@ def repeat(self, values, axis=None):
 
     >>> a2d.repeat([30., 50.], axis="lon")  
     dimarray: 6 non-null elements (0 null)
-    dimensions: 'time', 'lon'
     0 / time (3): 1950.0 to 1952.0
     1 / lon (2): 30.0 to 50.0
     array([[0, 0],
@@ -259,19 +308,25 @@ def repeat(self, values, axis=None):
     newaxes[idx] = newaxis
 
     # Update values and axes
-    return self._constructor(newvalues, newaxes, **self._metadata)
+    return self._constructor(newvalues, newaxes, **self.attrs)
 
 
 def newaxis(self, name, values=None, pos=0):
-    """ add a new axis, ready to broadcast
+    """ add a new axis
+
+    Add a singleton axis to ease broadcasting, 
+    and repeat array along this axis if required.
 
     Parameters
     ----------
-    name: `str`, axis name
-    values: optional, if provided, broadcast the array along the new axis
+    name : str
+        axis name
+    values: array-like, optional
+            if provided, broadcast the array along the new axis
             call `repeat(name, values)` after inserting the new singleton 
             dimension (see `repeat`) for more information.
-    pos: `int`, optional: axis position, default 0 (first axis)
+    pos : int, optional
+        axis position, default 0 (first axis)
 
     Returns
     -------
@@ -296,25 +351,22 @@ def newaxis(self, name, values=None, pos=0):
     >>> a = DimArray([1,2])
     >>> a
     dimarray: 2 non-null elements (0 null)
-    dimensions: 'x0'
     0 / x0 (2): 0 to 1
     array([1, 2])
     >>> a.newaxis('new', pos=1)
     dimarray: 2 non-null elements (0 null)
-    dimensions: 'x0', 'new'
     0 / x0 (2): 0 to 1
     1 / new (1): None to None
     array([[1],
            [2]])
     >>> a.newaxis('new', values=['a','b'],pos=1)
     dimarray: 4 non-null elements (0 null)
-    dimensions: 'x0', 'new'
     0 / x0 (2): 0 to 1
-    1 / new (2): a to b
+    1 / new (2): 'a' to 'b'
     array([[1, 1],
            [2, 2]])
     """
-    assert isinstance(name, str), "name must be string"
+    assert isinstance(name, basestring), "name must be string"
     if name in self.dims:
         raise ValueError("dimension already present: "+name)
 
@@ -332,7 +384,7 @@ def newaxis(self, name, values=None, pos=0):
     axes.insert(pos, axis)
 
     # create new object
-    newobj = self._constructor(newvalues, axes, **self._metadata)
+    newobj = self._constructor(newvalues, axes, **self.attrs)
 
     # If values is provided, repeat the array along values
     if values is not None:
@@ -344,7 +396,19 @@ def newaxis(self, name, values=None, pos=0):
 # Remove / add singleton axis
 #
 def squeeze(self, axis=None):
-    """ Analogous to numpy, but also allows axis name
+    """ Squeeze singleton axes
+    
+    Analogous to numpy, but also allows axis name
+
+    Parameters
+    ----------
+    axis : int or str or None 
+        axis to squeeze
+        default is None, to remove all singleton axes
+
+    Returns
+    -------
+    squeezed_array : DimArray
 
     Examples
     --------
@@ -352,19 +416,16 @@ def squeeze(self, axis=None):
     >>> a = da.DimArray([[[1,2,3]]])
     >>> a
     dimarray: 3 non-null elements (0 null)
-    dimensions: 'x0', 'x1', 'x2'
     0 / x0 (1): 0 to 0
     1 / x1 (1): 0 to 0
     2 / x2 (3): 0 to 2
     array([[[1, 2, 3]]])
     >>> a.squeeze()
     dimarray: 3 non-null elements (0 null)
-    dimensions: 'x2'
     0 / x2 (3): 0 to 2
     array([1, 2, 3])
     >>> a.squeeze(axis='x1')
     dimarray: 3 non-null elements (0 null)
-    dimensions: 'x0', 'x2'
     0 / x0 (1): 0 to 0
     1 / x2 (3): 0 to 2
     array([[1, 2, 3]])
@@ -378,9 +439,9 @@ def squeeze(self, axis=None):
         res = self.values.squeeze(idx)
         newaxes = [ax for ax in self.axes if ax.name != name or ax.size != 1] 
 
-    return self._constructor(res, newaxes, **self._metadata)
+    return self._constructor(res, newaxes, **self.attrs)
 
-def _ungroup_dims(dims):
+def _unflatten_dims(dims):
     """ ['a','b,c','d'] ==> ['a','b','c','d']
     """
     flatdims = []
@@ -393,26 +454,27 @@ def _ungroup_dims(dims):
 # Reshape by adding/removing as many singleton axes as needed to match prescribed dimensions
 #
 def reshape(self, *newdims, **kwargs):
-    """ Add/remove/group dimensions to conform array to new dimensions
+    """ Add/remove/flatten dimensions to conform array to new dimensions
     
     Parameters
     ----------
-        newdims: tuple or list or variable list of dimension names {str} 
+    newdims : tuple or list or variable list of dimension names {str} 
+        Any dimension now present in the array is added as singleton dimension
+        Any dimension name containing a comma is interpreting as a flattening command.
+        All dimensions to flatten have to exist already.
 
-            Any dimension now present in the array is added as singleton dimension
-
-            Any dimension name containing a comma is interpreting as a grouping command.
-            All dimensions to group have to exist already.
-
-        transpose: {bool} if True, transpose dimensions to match new order (default True)
+    transpose : bool
+        if True, transpose dimensions to match new order (default True)
+        otherwise, raise and Error if tranpose is needed (closer to original numpy's behaviour)
 
     Returns
     -------
-        reshape: DimArray with reshape.dims == tuple(newdims)
+    reshaped_array : DimArray 
+        with reshaped_array.dims == tuple(newdims)
 
     See also
     --------
-    group, ungroup, transpose, newaxis
+    flatten, unflatten, transpose, newaxis
 
     Examples
     --------
@@ -420,13 +482,11 @@ def reshape(self, *newdims, **kwargs):
     >>> a = DimArray([7,8])
     >>> a
     dimarray: 2 non-null elements (0 null)
-    dimensions: 'x0'
     0 / x0 (2): 0 to 1
     array([7, 8])
 
     >>> a.reshape(('x0','new'))
     dimarray: 2 non-null elements (0 null)
-    dimensions: 'x0', 'new'
     0 / x0 (2): 0 to 1
     1 / new (1): None to None
     array([[7],
@@ -435,7 +495,6 @@ def reshape(self, *newdims, **kwargs):
     >>> b = DimArray(np.arange(2*2*2).reshape(2,2,2))
     >>> b
     dimarray: 8 non-null elements (0 null)
-    dimensions: 'x0', 'x1', 'x2'
     0 / x0 (2): 0 to 1
     1 / x1 (2): 0 to 1
     2 / x2 (2): 0 to 1
@@ -448,7 +507,6 @@ def reshape(self, *newdims, **kwargs):
     >>> c = b.reshape('x0','x1,x2')
     >>> c
     dimarray: 8 non-null elements (0 null)
-    dimensions: 'x0', 'x1,x2'
     0 / x0 (2): 0 to 1
     1 / x1,x2 (4): (0, 0) to (1, 1)
     array([[0, 1, 2, 3],
@@ -456,7 +514,6 @@ def reshape(self, *newdims, **kwargs):
 
     >>> c.reshape('x0,x1','x2')
     dimarray: 8 non-null elements (0 null)
-    dimensions: 'x0,x1', 'x2'
     0 / x0,x1 (4): (0, 0) to (1, 1)
     1 / x2 (2): 0 to 1
     array([[0, 1],
@@ -480,46 +537,46 @@ def reshape(self, *newdims, **kwargs):
     # check that newd dimensions
     assert len(newdims) == len(set(newdims)), "must not contain duplicate axes !"
 
-    # First ungroup the array to compare with flattened newdims
-    o = self.ungroup()
+    # First unflatten the array to compare with flattened newdims
+    o = self.unflatten()
 
-    # Temporarily replace "," by ";" in any dimension with is NOT a grouped axis, and flatten all dimensions apart from that
+    # Temporarily replace "," by ";" in any dimension with is NOT a flattened axis, and flatten all dimensions apart from that
     newdims_renamed = []
     for d in newdims:
         if ',' in d and d in o.dims:
             d = d.replace(',',';')
         newdims_renamed.append(d)
-    newdims_ungrouped = _ungroup_dims(newdims_renamed) 
+    newdims_unflattened = _unflatten_dims(newdims_renamed) 
 
-    assert len(newdims_ungrouped) == len(set(newdims_ungrouped)), "must not contain duplicate axes !"
+    assert len(newdims_unflattened) == len(set(newdims_unflattened)), "must not contain duplicate axes !"
 
     for ax in o.axes:
         ax.name = ax.name.replace(',',';')
 
     # Remove unwanted singleton dimensions, if any
     for dim in o.dims:
-        assert isinstance(dim, str), "newdims must be a tuple of axis names (`str`)"
-        if dim not in newdims_ungrouped:
+        assert isinstance(dim, basestring), "newdims must be a tuple of axis names (`str`)"
+        if dim not in newdims_unflattened:
             o = o.squeeze(dim)
 
     # Transpose array to match existing dimensions
     if transpose:
-        o = o.transpose([dim for dim in newdims_ungrouped if dim in o.dims])
+        o = o.transpose([dim for dim in newdims_unflattened if dim in o.dims])
 
     # check sortedness
     else:
-        if tuple([d for d in o.dims if d in newdims_ungrouped]) != tuple([d for d in newdims_ungrouped if d in o.dims]):
+        if tuple([d for d in o.dims if d in newdims_unflattened]) != tuple([d for d in newdims_unflattened if d in o.dims]):
             raise ValueError("First transpose the array before reshaping, or set parameter `transpose=True`")
 
     # Add missing dimensions by inserting singleton axes
-    for i, dim in enumerate(newdims_ungrouped):
+    for i, dim in enumerate(newdims_unflattened):
         if dim not in o.dims:
             o = o.newaxis(dim, pos=i)
 
     # Group dimensions as required
     for i, d in enumerate(newdims_renamed):
         if ',' in d:
-            o = o.group(d.split(','), insert=i)
+            o = o.flatten(d.split(','), insert=i)
 
     # Replace back ';' by ','
     for ax in o.axes:
@@ -532,27 +589,35 @@ def reshape(self, *newdims, **kwargs):
 
 
 #
-# Group/ungroup subsets of axes to perform operations on partly flattened array
+# Group/unflatten subsets of axes to perform operations on partly flattened array
 #
-def group(self, *dims, **kwargs):
-    """ group (or flatten) a subset of dimensions
+def flatten(self, *dims, **kwargs):
+    """Flatten all or a subset of dimensions
 
     Parameters
     ----------
-        - dims: list or tuple of axis names
-        - reverse [False]: if True, reverse behaviour: dims are interpreted as 
-            the dimensions to keep, and all the other dimensions are grouped
-        - insert, optional: position where to insert the grouped axis 
-                  (by default, any grouped dimension is inserted at 
-                the position of the first axis involved in grouping)
+    dims : list or tuple of axis names, optional
+        by default, all dimensions
+    reverse : bool, optional
+        if True, reverse behaviour: dims are interpreted as 
+        the dimensions to keep, and all the other dimensions are flattened
+        default is False
+    insert : int, optional
+        position where to insert the flattened axis 
+        (by default, any flattened dimension is inserted at 
+        the position of the first axis involved in flattening)
 
     Returns
     -------
-        - DimArray appropriately reshaped, with collapsed dimensions as first axis (tuples)
+    flattened_array : DimArray
+        appropriately reshaped, with collapsed dimensions as first axis (tuples)
 
     This is useful to do a regional mean with missing values
 
-    Note: can be passed via the "axis" parameter of the transformation, too
+    Notes
+    -----
+    A tuple of axis names can be passed via the "axis" parameter of the transformation
+    to trigger flattening prior to reducing an axis.
 
     See also
     --------
@@ -560,13 +625,35 @@ def group(self, *dims, **kwargs):
 
     Examples
     --------
-    >>> import dimarray as da
+
+    Flatten all dimensions 
+
+    >>> from dimarray import DimArray
+    >>> a = DimArray([[1,2,3],[4,5,6]])
+    >>> a
+    dimarray: 6 non-null elements (0 null)
+    0 / x0 (2): 0 to 1
+    1 / x1 (3): 0 to 2
+    array([[1, 2, 3],
+           [4, 5, 6]])
+
+    >>> b = a.flatten()
+    >>> b
+    dimarray: 6 non-null elements (0 null)
+    0 / x0,x1 (6): (0, 0) to (1, 2)
+    array([1, 2, 3, 4, 5, 6])
+
+    >>> b.labels
+    (array([(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2)], dtype=object),)
+
+    Flatten a subset of dimensions only
+
+    >>> from dimarray import DimArray
     >>> np.random.seed(0)
     >>> values = np.arange(2*3*4).reshape(2,3,4)
-    >>> v = da.array_kw(values, time=[1950,1955], lat=np.linspace(-90,90,3), lon=np.linspace(-180,180,4))
+    >>> v = DimArray(values, axes=[('time', [1950,1955]), ('lat', np.linspace(-90,90,3)), ('lon', np.linspace(-180,180,4))])
     >>> v
     dimarray: 24 non-null elements (0 null)
-    dimensions: 'time', 'lat', 'lon'
     0 / time (2): 1950 to 1955
     1 / lat (3): -90.0 to 90.0
     2 / lon (4): -180.0 to 180.0
@@ -578,23 +665,21 @@ def group(self, *dims, **kwargs):
             [16, 17, 18, 19],
             [20, 21, 22, 23]]])
 
-    >>> w = v.group(('lat','lon'), insert=1)
+    >>> w = v.flatten(('lat','lon'), insert=1)
     >>> w 
     dimarray: 24 non-null elements (0 null)
-    dimensions: 'time', 'lat,lon'
     0 / time (2): 1950 to 1955
     1 / lat,lon (12): (-90.0, -180.0) to (90.0, 180.0)
     array([[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11],
            [12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]])
 
-    >>> np.all( w.ungroup() == v )
+    >>> np.all( w.unflatten() == v )
     True
 
     But be careful, the order matter !
 
-    >>> v.group(('lon','lat'), insert=1)
+    >>> v.flatten(('lon','lat'), insert=1)
     dimarray: 24 non-null elements (0 null)
-    dimensions: 'time', 'lon,lat'
     0 / time (2): 1950 to 1955
     1 / lon,lat (12): (-180.0, -90.0) to (180.0, 90.0)
     array([[ 0,  4,  8,  1,  5,  9,  2,  6, 10,  3,  7, 11],
@@ -602,9 +687,8 @@ def group(self, *dims, **kwargs):
 
     Useful to average over a group of dimensions:
 
-    >>> v.group(('lon','lat'), insert=0).mean(axis=0)
+    >>> v.flatten(('lon','lat'), insert=0).mean(axis=0)
     dimarray: 2 non-null elements (0 null)
-    dimensions: 'time'
     0 / time (2): 1950 to 1955
     array([  5.5,  17.5])
 
@@ -612,7 +696,6 @@ def group(self, *dims, **kwargs):
 
     >>> v.mean(axis=('lon','lat')) 
     dimarray: 2 non-null elements (0 null)
-    dimensions: 'time'
     0 / time (2): 1950 to 1955
     array([  5.5,  17.5])
     """
@@ -630,10 +713,10 @@ def group(self, *dims, **kwargs):
             reorder = True
 
     if len(dims) == 0: 
-        raise ValueError("dims must have length > 0")
+        dims = self.dims
 
     if type(dims) not in (tuple, list, set):
-        raise TypeError("dimensions to group must be a list or a tuple  or a set")
+        raise TypeError("dimensions to flatten must be a list or a tuple  or a set")
 
     # make sure dims contains axis names
     _ , dims = self._get_axes_info(dims)
@@ -643,12 +726,12 @@ def group(self, *dims, **kwargs):
     if reverse:
         dims = [d for d in self.dims if d not in dims]
 
-    # reorder dimensions to group and convert to tuple
+    # reorder dimensions to flatten and convert to tuple
     if reorder: # only if provided as set
         dims = [d for d in self.dims if d in dims] # do not reorder, this may matter
     dims = tuple(dims)
 
-    n = len(dims) # number of dimensions to group
+    n = len(dims) # number of dimensions to flatten
     ii = self.dims.index(dims[0]) # start
 
     # dimension to insert the new axis at
@@ -664,10 +747,10 @@ def group(self, *dims, **kwargs):
     
         b = self.transpose(newdims) # dimensions to factorize in the front
 
-        return b.group(dims, insert=insert)
+        return b.flatten(dims, insert=insert)
 
-    # Create a new grouped axis
-    newaxis = GroupedAxis(*[ax for ax in self.axes if ax.name in dims])
+    # Create a new flattened axis
+    newaxis = MultiAxis(*[ax for ax in self.axes if ax.name in dims])
 
     # New axes
     newaxes = [ax for ax in self.axes if ax.name not in dims]
@@ -678,58 +761,32 @@ def group(self, *dims, **kwargs):
     newvalues = self.values.reshape(newshape)
 
     # Define the new array
-    new = self._constructor(newvalues, newaxes, **self._metadata)
+    new = self._constructor(newvalues, newaxes, **self.attrs)
 
     return new
 
-def flatten(self):
-    """ analogous to numpy's flatten, but conserves axes with a GroupedAxis object
+group = deprecated_func(flatten, 'group')
 
-    See also
-    --------
-    reshape, group, ungroup
-
-    Examples
-    --------
-    >>> from dimarray import DimArray
-    >>> a = DimArray([[1,2,3],[4,5,6]])
-    >>> a
-    dimarray: 6 non-null elements (0 null)
-    dimensions: 'x0', 'x1'
-    0 / x0 (2): 0 to 1
-    1 / x1 (3): 0 to 2
-    array([[1, 2, 3],
-           [4, 5, 6]])
-
-    >>> b = a.flatten()
-    >>> b
-    dimarray: 6 non-null elements (0 null)
-    dimensions: 'x0,x1'
-    0 / x0,x1 (6): (0, 0) to (1, 2)
-    array([1, 2, 3, 4, 5, 6])
-
-    >>> b.labels
-    (array([(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2)], dtype=object),)
-    """
-    return self.group(self.dims)
-
-def ungroup(self, axis=None):
-    """ opposite from group
+def unflatten(self, axis=None):
+    """ undo flatten (inflate array)
 
     Parameters
     ----------
-    axis: axis to ungroup as int or str (default: ungroup all)
+    axis : int or str or None, optional
+        axis to unflatten
+        default to None to unflatten all
 
     Returns
     -------
     DimArray
+
     """
-    # by default, ungroup all
+    # by default, unflatten all
     if axis is None:
-        grouped_axes = [ax.name for ax in self.axes if isinstance(ax, GroupedAxis)]
+        flattened_axes = [ax.name for ax in self.axes if isinstance(ax, MultiAxis)]
         obj = self
-        for axis in grouped_axes:
-            obj = obj.ungroup(axis=axis)
+        for axis in flattened_axes:
+            obj = obj.unflatten(axis=axis)
         return obj
 
     assert type(axis) in (str, int), "axis must be integer or string"
@@ -737,13 +794,15 @@ def ungroup(self, axis=None):
     group = self.axes[axis]    # axis to expand
     axis = self.dims.index(group.name) # make axis be an integer
 
-    assert isinstance(group, GroupedAxis), "can only ungroup a GroupedAxis"
+    assert isinstance(group, MultiAxis), "can only unflatten a MultiAxis"
 
     newshape = self.shape[:axis] + tuple(ax.size for ax in group.axes) + self.shape[axis+1:]
     newvalues = self.values.reshape(newshape)
     newaxes = self.axes[:axis] + group.axes + self.axes[axis+1:]
 
-    return self._constructor(newvalues, newaxes, **self._metadata)
+    return self._constructor(newvalues, newaxes, **self.attrs)
+
+ungroup = deprecated_func(unflatten, 'ungroup')
 
 #
 # groupby method
@@ -802,31 +861,32 @@ class GroupBy(object):
 #for op in _trans:
 #    GroupBy.__dict__[op] = Desc(op, axis=-1)
 
-def groupby(self, *dims):
-    """ group by one or several variables along which stat functions can be applied
-
-    Parameters
-    ----------
-        *dims: variable list of dims to keep, all others are flattened
-
-    Returns
-    -------
-        GroupBy object
-
-    Notes 
-    -----
-    this method is experimental and may change in the future
-
-    Examples
-    --------
-    >>> from dimarray import DimArray
-    >>> a = DimArray([[1,2,3],[4,5,6]], [('x',[1,2]), ('items',['a','b','c'])])
-    >>> a = a.newaxis('y',4) # add an axis of length 4
-    >>> a.groupby('items').mean()
-    dimarray: 3 non-null elements (0 null)
-    dimensions: 'items'
-    0 / items (3): a to c
-    array([ 2.5,  3.5,  4.5])
-    """
-    obj = group(self, dims, reverse=True, insert=0).T
-    return GroupBy(obj, dims)
+# def groupby(self, *dims):
+#     """ group by one or several variables along which stat functions can be applied
+#
+#     .. note:: EXPERIMENTAL: will probably be removed or renamed in future releases
+#
+#     Parameters
+#     ----------
+#     *dims: variable list of dims to keep, all others are flattened
+#
+#     Returns
+#     -------
+#     GroupBy object
+#
+#     Notes 
+#     -----
+#     this method is experimental and may change in the future
+#
+#     Examples
+#     --------
+#     >>> from dimarray import DimArray
+#     >>> a = DimArray([[1,2,3],[4,5,6]], [('x',[1,2]), ('items',['a','b','c'])])
+#     >>> a = a.newaxis('y',4) # add an axis of length 4
+#     >>> a.groupby('items').mean()
+#     dimarray: 3 non-null elements (0 null)
+#     0 / items (3): 'a' to 'c'
+#     array([ 2.5,  3.5,  4.5])
+#     """
+#     obj = group(self, dims, reverse=True, insert=0).T
+#     return GroupBy(obj, dims)
